/*
 * I2C.cpp
 *
 *  Created on: 11-11-2013
 *      Author: Wojtek
 */

#include "I2C.h"
int8_t I2C::readFromReg8(uint8_t devAddr, uint8_t regAddr, uint8_t *data) {
	return I2C::readFromReg(devAddr, regAddr, data, 1);

}

int8_t I2C::readFromReg(uint8_t devAddr, uint8_t regAddr, uint8_t *data,
uint8_t length) {

	int8_t count = 0;
	int fd = open(I2C_DEV, O_RDWR);

	if (fd < 0) {
		fprintf(stderr, "Failed to open device: %s\n", strerror(errno));
		return 0;
	}
	if (ioctl(fd, I2C_SLAVE, devAddr) < 0) {
		fprintf(stderr, "Failed to select device: %s\n", strerror(errno));
		close(fd);
		return (-1);
	}
	if (write(fd, &regAddr, 1) != 1) {
		fprintf(stderr, "Failed to write reg: %s\n", strerror(errno));
		close(fd);
		return (-1);
	}
	count = read(fd, data, length);

	if (count < 0) {
		fprintf(stderr, "Failed to read device(%d): %s\n", count,
				::strerror(errno));
		close(fd);
		return (-1);
	} else if (count != length) {
		fprintf(stderr, "Short read  from device, expected %d, got %d\n",
				length, count);
		close(fd);
		return (-1);
	}

	close(fd);

	return length;
}

bool I2C::writeToReg8(uint8_t devAddr, uint8_t regAddr, uint8_t data) {
	I2C::writeToReg(devAddr, regAddr, &data, 1);
}

bool I2C::writeToReg(uint8_t devAddr, uint8_t regAddr, uint8_t *data,
		uint8_t length) {
	int8_t count = 0;
	uint8_t buf[128];
	int fd;

	if (length > 127) {
		fprintf(stderr, "Byte write count (%d) > 127\n", length);
		return false;
	}

	fd = open(I2C_DEV, O_RDWR);
	if (fd < 0) {
		fprintf(stderr, "Failed to open device: %s\n", strerror(errno));
		return false;
	}
	if (ioctl(fd, I2C_SLAVE, devAddr) < 0) {
		fprintf(stderr, "Failed to select device: %s\n", strerror(errno));
		close(fd);
		return false;
	}
	buf[0] = regAddr;
	memcpy(buf + 1, data, length);
	count = write(fd, buf, length + 1);
	if (count < 0) {
		fprintf(stderr, "Failed to write device(%d): %s\n", count,
				::strerror(errno));
		close(fd);
		return false;
	} else if (count != length + 1) {
		fprintf(stderr, "Short write to device, expected %d, got %d\n",
				length + 1, count);
		close(fd);
		return false;
	}
	close(fd);

	return true;
}
